/*
 * Copyright 2010, 2011, 2012, 2013 mapsforge.org
 *
 * This program is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.mapsforge.core.graphics;

public enum FontFamily {
    DEFAULT, MONOSPACE, SANS_SERIF, SERIF;

    public static FontFamily fromString(String value) {
        if ("default".equals(value)) {
            return DEFAULT;
        } else if (("monospace").equals(value)) {
            return MONOSPACE;
        } else if ("sans_serif".equals(value)) {
            return SANS_SERIF;
        } else if ("serif".equals(value)) {
            return SERIF;
        }
        throw new IllegalArgumentException("Invalid value for FontFamily: " + value);
    }
}
