/*
 * Copyright 2010, 2011, 2012, 2013 mapsforge.org
 *
 * This program is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.mapsforge.map.layer.renderer;

import org.mapsforge.core.graphics.Curve;
import org.mapsforge.core.graphics.Paint;

public class ShapePaintContainer {
    final float dy;
    final Paint paint;
    final ShapeContainer shapeContainer;
    final Curve curveStyle;

    public ShapePaintContainer(ShapeContainer shapeContainer, Paint paint) {
        this(shapeContainer, paint, 0f, Curve.NO);
    }

    public ShapePaintContainer(ShapeContainer shapeContainer, Paint paint, float dy, Curve curveStyle) {
        this.shapeContainer = shapeContainer;
        this.paint = paint;
        this.dy = dy;
        this.curveStyle = curveStyle;
    }
}
