/*
 * Copyright 2010, 2011, 2012, 2013 mapsforge.org
 * Copyright 2022 devemux86
 *
 * This program is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package org.mapsforge.samples.awt;

import org.junit.Assert;
import org.junit.Test;

public class SamplesTest {
    private static void verifyInvalidArguments(String[] args) {
        try {
            Samples.main(args);
            Assert.fail();
        } catch (IllegalArgumentException e) {
            Assert.assertTrue(true);
        }
    }

    @Test
    public void mainTest() {
        String[] args = new String[]{};
        verifyInvalidArguments(args);

        /*args = new String[]{"file/not/found.map"};
        verifyInvalidArguments(args);*/
    }
}
