/*
 * Copyright 2010, 2011, 2012, 2013 mapsforge.org
 *
 * This program is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * The mapsforge-map-writer tool comes as a plugin for <a href="http://wiki.openstreetmap.org/wiki/Osmosis">osmosis</a>.
 * A detailed documentation of the installation procedure and its use is provided
 * on the mapsforge project <a href="https://github.com/mapsforge/mapsforge/blob/master/docs/Getting-Started-Map-Writer.md">website</a>.
 * <p/>
 * This software is a part of the <a href="http://mapsforge.org/" target="_top">mapsforge</a> project and
 * distributed under the <a href="http://www.gnu.org/licenses/lgpl.html" target="_top">LGPL3 license</a>. All
 * map data (c) <a href="http://www.openstreetmap.org/" target="_top">OpenStreetMap</a> contributors, <a
 * href="http://creativecommons.org/licenses/by-sa/2.0/" target="_top">CC-BY-SA license</a>.
 */
package org.mapsforge.map.writer;
